package cn.com.qmth.examcloud.service.core.api;

import io.swagger.annotations.ApiOperation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Course;
import cn.com.qmth.examcloud.core.basic.dao.entity.Specialty;
import cn.com.qmth.examcloud.core.basic.service.impl.CourseSpeciatlyService;

/**
 * @Description 课程专业关联API
 * @author Administrator
 * @Date  2017年8月8号
 */
@RestController
@RequestMapping("${app.api.root}/CourseSpeciatly")
public class CourseSpeciatlyApi {

	@Autowired
	CourseSpeciatlyService courseSpeciatlyService;
	
	@Autowired
	CourseRepo courseRepo;
	
	@Autowired
	SpecialtyRepo specialtyRepo;
	
	/**
	 * 根据专业ID查询课程
	 * @param speciatlyId
	 * @return
	 */
	@ApiOperation(value="根据专业ID取课程",notes="根据专业ID取课程")
	@GetMapping("/allCourses/{speciatlyId}")
	public ResponseEntity getAllCoursesBySpeciatly(@PathVariable String speciatlyId,HttpServletRequest request){
		AccessUser  accessUser = (AccessUser)request.getAttribute("accessUser");
		List<Course> courses = courseSpeciatlyService.getAllCoursesBySpeciatlyId(speciatlyId);
		//List<Course> list = courseSpeciatlyService.getCoursesNotInSpeciatly(courses);
		List<Course> list = courseRepo.findByOrgId(accessUser.getRootOrgId());
		Map map = new HashMap();
		map.put("courseList", courses);
		map.put("courseAllList", list);
		return new ResponseEntity(map,HttpStatus.OK);
	}
	
	@ApiOperation(value="新增关联课程专业",notes="新增关联课程专业")
	@PostMapping("/addCourse/{speciallyId}")
	public ResponseEntity addCourseSpeciatly(@RequestBody List<String> list,@PathVariable String speciallyId,HttpServletRequest request){
		AccessUser  accessUser = (AccessUser)request.getAttribute("accessUser");
		courseSpeciatlyService.addCourseSpecialty(accessUser.getUserId(), list, speciallyId);
		return new ResponseEntity(HttpStatus.OK);
	}
	
	@ApiOperation(value="根据课程ID取专业",notes="根据课程ID取专业")
	@GetMapping("/allSpecialty/{courseId}")
	public ResponseEntity getAllSpeciatly(@PathVariable Long courseId,HttpServletRequest request){
		AccessUser  accessUser = (AccessUser)request.getAttribute("accessUser");
		List<Specialty> specialtys = courseSpeciatlyService.getAllSpecialtyByCourseId(courseId);
		List<Specialty> list = specialtyRepo.findByOrgId(accessUser.getRootOrgId());
		Map map = new HashMap();
		map.put("specialtyList", specialtys);
		map.put("specialtyAllList", list);
		return new ResponseEntity(map,HttpStatus.OK);
	}
	
	@ApiOperation(value="新增关联课程专业",notes="新增关联课程专业")
	@PostMapping("/addSpeciatly/{courseId}")
	public ResponseEntity addSpeciatly(@RequestBody List<String> list,@PathVariable Long courseId,HttpServletRequest request){
		AccessUser  accessUser = (AccessUser)request.getAttribute("accessUser");
		courseSpeciatlyService.addCourse(accessUser.getUserId(), list, courseId);
		return new ResponseEntity(HttpStatus.OK);
	}
}
